-- only load if optional deps are present
if not AceLibrary or not FuBar then
	return
end

LeeroyFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0","AceDB-2.0")
if not LeeroyFu then
	return
end

LeeroyFu.title = "LeeroyFu"
LeeroyFu.name = "LeeroyHillCatsPower"
LeeroyFu.category = "Audio"
LeeroyFu.hasIcon = "Interface\\AddOns\\LeeroyHillCatsPower\\LHCPIcon.tga"
LeeroyFu.cannotDetachTooltip = true

LeeroyFu:RegisterDB("LeeroyFuDB")

LeeroyFu:RegisterDefaults('profile', {
	showHeaderText = true,
})

-- TODO will this restore missing show/hide text&icon options?
--LeeroyFu.overrideMenu = true
--LeeroyFu:AddImplicitMenuOptions()

LeeroyFu.LHCP_emoteMenu = {}
LeeroyFu.LHCP_previewMenu = {}

LeeroyFu.OnMenuRequest = {
	type = 'group',
	args = {
		headerText = {
			type = 'toggle',
			name = "Show header text",
			desc = "Show header text",
			get = "IsShowingHeaderText",
			set = "ToggleShowingHeaderText",
			order = -15,
		},
		emote = {
			type = 'group',
			name = "Emote...",
			desc = "Broadcast a LHCP emote!",
			args = LeeroyFu.LHCP_emoteMenu,
			order = 10,
		},
--		private = {
--			type = 'group',
--			name = "Preview...",
--			desc = "Privately play a LHCP sound!",
--			args = LeeroyFu.LHCP_emoteMenu,
--			order = 20,
--		},
		togglespacer = {type = 'header', order = 30},
		raidignore = {
			type = 'toggle',
			name = "Ignore in raid",
			desc = "Ignore LHCP when in a raid instance",
			get = "IsRaidIgnore",
			set = "ToggleRaidIgnore",
			order = 40,
		},
		pvpignore = {
			type = 'toggle',
			name = "PvP ignore",
			desc = "Ignore emotes from the opposing faction",
			get = "IsPvpIgnore",
			set = "TogglePvpIgnore",
			order = 50,
		},
		nsfw = {
			type = 'toggle',
			name = "Play NSFW",
			desc = "Play not safe for work sounds",
			get = "IsNsfw",
			set = "ToggleNsfw",
			order = 60,
		},
	}
}

local tablet = AceLibrary("Tablet-2.0")

function LeeroyFu:IsShowingHeaderText()
	return self.db.profile.showHeaderText
end

function LeeroyFu:GetHeaderText()
	if self:IsShowingHeaderText() then
		return "LHCP! "
	else
		return ""
	end
end

function LeeroyFu:ToggleShowingHeaderText()
	self.db.profile.showHeaderText = not self.db.profile.showHeaderText
	self:UpdateText()
end

function LeeroyFu:OnTooltipUpdate()
	tablet:SetHint("Double-click to toggle basic ignore mode.");
end

function LeeroyFu:IsPvpIgnore()
	return not LHCP_PlayEnemy()
end

function LeeroyFu:TogglePvpIgnore()
	LHCP_pvp(true)
end

function LeeroyFu:IsRaidIgnore()
	return LHCP_RaidIgnore()
end

function LeeroyFu:ToggleRaidIgnore()
	LHCP_ToggleRaidIgnore(true);
end

function LeeroyFu:IsNsfw()
	return LHCP_PlayNsfw()
end

function LeeroyFu:ToggleNsfw()
	if self:IsNsfw() then
		LHCP_Sfw(true)
	else
		LHCP_Nsfw(true)
	end
end

-- TODO how? docs tell me to self.frame:RegisterForClicks("LeftButton")
function LeeroyFu:OnDoubleClick(button)
	if button == "LeftButton" then
		-- cycle ignore
		LHCP_ToggleIgnore(true) -- toggle ignore calls LeeroyFu:UpdateText()
	end
end

function LeeroyFu:OnTextUpdate()
	if self:IsTextShown() then
		self:SetText(self:GetHeaderText()..LHCP_GetStatus())
	end
end

function LeeroyFu:LHCP_Setup(data,categories)
	LeeroyFu.LHCP_categories = categories
	LeeroyFu.LHCP_data = data
	
	local menu = LeeroyFu.LHCP_emoteMenu
	
	-- TODO these give Error: Dewdrop-2.0: Handler "nil" not available
--	Play = function(key)
--		-- play outgoing emote
--		LHCP_SayOutgoingEmote(key)
--	end
--	
--	local IsDisabled = function(key)
--		-- check for NSFW flag (and throttle)
--		return LHCP_IsThrottled() or LHCP_IsCensored(key)
--	end
--	
--	local PrivPlay = function(key)
--		-- call private play LHCP emote
--		LHCP_PrivatePlay(key)
--	end
	
	local _G = getfenv(0);

	for k,v in pairs(categories) do
		if v.data and getn(v.data) > 0 then
			local list = {}
			
			list.random = {
				type = 'execute',
				desc = "Play random "..k.." emote",
				name = "random",
				func = loadstring("LHCP_Random(\""..k.."\")"),
				order = 1,
			}
			list.rheader = {type = 'header', order = 2}
			
			for _,emote in ipairs(v.data) do
				local txt = data[emote].text
				  or "-- ERROR in "..emote.." entry --"
				local fn = _G.SlashCmdList["LHCP"..emote]
				  or loadstring("LHCP_SayOutgoingEmote(\""
				    ..emote.."\")")
				
				list[emote] = {
					type = 'execute',
					desc = txt,
					name = emote,
					func = fn,
				}
			end
			table.sort(list)
			
			menu[k] = {
				type = 'group',
				desc = v.help or k,
				name = k,
--				pass = true, -- gives nil handler error
--				func = Play,
--				disabled = IsDisabled,
				args = list,
--				handler = self,
			}
		end

		menu.random = {
			type = 'execute',
			desc = "Play random emote",
			name = "random",
			func = loadstring("LHCP_Random()"),
			order = 1,
		}
		menu.rheader = {type = 'header', order = 2}
	end
	table.sort(menu)
end

