-- Leeroy Hill Cats Power Extended
-- Written by Barogio of Thunderhorn with help from Moomin!
-- Based on Benny Hill mod by Catalyst + Springbeart of Thunderhorn
-- Extended and Optimized by Ruuro and Basic of Kil'Jaeden

local _G = getfenv(0);		-- global hook
local _;

LHCP_IGNORE_SPAM = 2^0;		-- ignore repeated emotes
LHCP_IGNORE_ENEMY = 2^1;	-- ignore opposing faction LHCP
LHCP_IGNORE_SOUNDS = 2^2;	-- don't play sounds
LHCP_IGNORE_EMOTES = 2^3;	-- don't fire emotes
LHCP_PLAY_NSFW = 2^4;		-- ignore NSFW LHCP
LHCP_IGNORE_RAID = 2^5;		-- ignore LHCP when in raid

LHCP_timeTilNext = 0;		-- throttle sound spamming
LHCP_emoteTime = 0;		-- throttle emote spamming
LHCP_globalIgnoreMode = false;  -- make prefs global
LHCP_ignoreMode =		-- ignore mode 
  bit.bor(LHCP_IGNORE_SPAM,LHCP_IGNORE_RAID);
LHCP_factionText = nil;		-- player's faction name
LHCP_isEnabled = false;		-- disable parsing when not in world
LHCP_debug = false;		-- print debug statements

LHCP_events = {			-- LHCP event function hooks
}

LHCP_eggs = {			-- LHCP easter egg hooks
}

LHCP_emoteList = {		-- Indexable list of all emotes
}

LHCP_phrases = nil;		-- LHCP phrase index


LHCP_skipEvent = nil;		-- event name to skip
LHCP_lastEmote = nil;		-- last emote, for emote antispam
LHCP_skipText = nil;		-- event text to skip

LHCP_actionList = nil;		-- list of LHCP actions for /lhcp list

LHCP_Orig_ChatFrame_OnEvent = nil;	-- non-Sea chat frame replacement


---
--- Init
---

---
-- LHCP_MakeSlashCmd	- register a slash command with WoW
-- name		- slash constant name
-- cmd		- slash command without '/'
-- i		- slash constant index
-- slashList	- set of existing slash commands with '/'
-- return	- false if cmd was already registered
---
function LHCP_MakeSlashCmd(name,cmd,i,slashList)
	cmd = "/"..cmd;
	
	if slashList and slashList[cmd] then
		return false;
	end
	
	_G["SLASH_"..name..i] = cmd;
	
	return true;
end

---
-- LHCP_GetSlashList	- generate a set of registered slash commands
-- return	- set of /cmd = key (where key is the index in the global SlashCmdList)
---
function LHCP_GetSlashList()
	local timeout = time() + 10;
	local slashList = {};
	for key,_ in pairs(_G.SlashCmdList) do
		local i = 1;
		local name = "SLASH_"..key..i;
		while _G[name] do
			slashList[_G[name]] = key;
			i = i+1;
			name = "SLASH_"..key..i;
		end
	end

	return slashList;
end

---
-- LHCP_Init	- register events, hook chat frame functions
-- CHAT_MSG_SAY			= LHCP_PlayIncomingText
-- CHAT_MSG_TEXT_EMOTE		= LHCP_HandleTextEmote
-- PLAYER_LEAVING_WORLD		= LHCP_Disable
-- PLAYER_ENTERING_WORLD	= LHCP_Enable
---
function LHCP_Init()
	-- need to register chat events here - because they are fired
	-- in the order they were registered
	
	-- register additional events
	LHCP_events["CHAT_MSG_SAY"] = LHCP_PlayIncomingText;
	LHCP_events["CHAT_MSG_TEXT_EMOTE"] = LHCP_HandleTextEmote;
	LHCP_events["PLAYER_LEAVING_WORLD"] = LHCP_Disable;
	LHCP_events["PLAYER_ENTERING_WORLD"] = LHCP_Enable;
	
	-- actually hook up all events
	LHCP_HookEvents();
	
	-- filter the chat frame
	LHCP_HookAllChatFrames();
	
	if LHCP_globalIgnoreMode then
		LHCP_ignoreMode = LHCP_globalIgnoreMode;
		LHCP_OnUpdateStatus()
	end
end

---
-- LHCP_Enable	- allow parsing of events
---
function LHCP_Enable()
	LHCP_isEnabled = true;
end

---
-- LHCP_Disable	- disable parsing of events
---
function LHCP_Disable()
	LHCP_isEnabled = false;
end

---
-- LHCP_LoadEEgg	- load easter egg
-- key		- easteregg key into the data table
-- value	- easteregg entry
-- abortTime	- loader timeout time
---
function LHCP_LoadEEgg(key,value,abortTime)
	if time() > abortTime then error("LHCP load stalled on load eegg "..key); end
	
	if value["event"] ~= nil
	  and value["text"] ~= nil
	  and value["file"] ~= nil then
	  	
	  	local ev = value["event"];
		
	  	if LHCP_eggs[ev] == nil then
			-- TODO this will not register properly!
			LHCP_events[ev] = LHCP_DoEgg;
			LHCP_eggs[ev] = {};
		end
	  	
		table.insert(LHCP_eggs[ev],key);
	end
end

---
--
---
function LHCP_LoadDataCategories(key,value,abortTime)
	if not LeeroyHillCatsPower_categories then return; end
	
	if time() > abortTime then error("LHCP load stalled on load category "..key); end
	
	if value["menu"] ~= nil then
		for i,name in ipairs(value["menu"]) do
			local cat = LeeroyHillCatsPower_categories[name];
			if not cat then
				cat = {["help"] = "Unknown category: "..name};
				LeeroyHillCatsPower_categories[name] = cat;
			end
			
			local data = cat["data"];
			if not data then
				data = {};
				cat["data"] = data;
			end

			table.insert(data,key);
		end
	end
end

---
-- LHCP_LoadDataText	- load LHCP text into phrase lookup table
-- key		- emote key into the data table
-- value	- emote entry
-- abortTime	- loader timeout time
---
function LHCP_LoadDataText(key,value,abortTime)
	if time() > abortTime then error("LHCP load stalled on text "..key); end
	
	if value["text"] ~= nil and value["file"] ~= nil then
		local txts = {
			value["text"],
			value[LHCP_factionText],
			value["alttext"]
		};
		
		for i,t in ipairs(txts) do
			if t ~= nil and string.len(t) > 0 then
				LHCP_phrases[t] = key;
			end
		end
	end
end

---
-- LHCP_LoadDataCmd	- load LHCP emote cmd
-- key		- emote key into the data table
-- value	- emote entry
-- abortTime	- loader timeout time
---
function LHCP_LoadDataCmd(key,value,abortTime)
	if time() > abortTime then error("LHCP load stalled on cmd "..key); end
	
	local name = "LHCP"..key;
	-- need to use a dynamic script here to avoid variable scoping
	-- how do you pre-evaluate "key" ???
	_G.SlashCmdList[name] = loadstring("LHCP_SayOutgoingEmote(\""..key.."\");");
	-- _G.SlashCmdList[name] = function() { local k = key; LHCP_SayOutgoingEmote(k); };
	
	local i = 1;
	_= LHCP_MakeSlashCmd(name,key,i,slashList)
	  or LHCP_Print("LHCP register /"..key.." failed",1,.3,.3);
	
	i = i+1;
	
	-- add slash aliases
	local j = 1;
	local cmd = value["cmd"..j];
	while cmd ~= nil do
		_= LHCP_MakeSlashCmd(name,cmd,i)
		  or LHCP_Print("LHCP register /"..cmd.." failed",1,.3,.3);
		
		i = i+1;
		j = j+1;
		cmd = value["cmd"..j];
	end
end

---
-- LHCP_Load		- load emote, easter eggs, and text entries

function LHCP_Load()
	local t0 = time();
	local abortTime = t0 + 10;
	
	local slashList = LHCP_GetSlashList();

	-- get player faction
	LHCP_factionText = UnitFactionGroup("player").."EnemyText";
	
	LHCP_phrases = {};
	
	-- generate all the slash commands!
	if LeeroyHillCatsPower_data then
		for key,value in pairs(LeeroyHillCatsPower_data) do
			LHCP_LoadDataCmd(key,value,abortTime);
			LHCP_LoadDataText(key,value,abortTime);
			LHCP_LoadDataCategories(key,value,abortTime);
			table.insert(LHCP_emoteList,key);
		end
	end

	-- sort categories
	if LHCP_LoadDataCategories then
		for key,value in pairs(LeeroyHillCatsPower_categories) do
			local data = value["data"];
			if data then
				table.sort(data);
			end
		end
		table.sort(LeeroyHillCatsPower_categories)

		if LeeroyFu then
			LeeroyFu:LHCP_Setup(LeeroyHillCatsPower_data,LeeroyHillCatsPower_categories)
		end
	end
	
	-- index the easter eggs
	if LeeroyHillCatsPower_eastereggs then
		for key,value in pairs(LeeroyHillCatsPower_eastereggs) do
			LHCP_LoadEEgg(key,value,abortTime);
		end
	end
	
	local dt = time() - t0;
	
	LHCP_Print("LeeroyHillCatsPower loaded. "
	  .."Type /lhcp for more info.",0,1,0);
	LHCP_Debug("LHCP load took ",t0," seconds.");
	
	LHCP_ShowStatus();
	
	-- TODO need to register easteregg events
	-- hey... they are not filtered! so we can hook/unhook them at any time
	
	LHCP_nextToggle = LHCP_toggle[LHCP_GetStatus()];
end

---
-- 
---
function LHCP_HookAllChatFrames()
	if Sea then
		LHCP_Debug("Using Sea library to hook ",
		  NUM_CHAT_WINDOWS," chat windows");
		
		for i=1,NUM_CHAT_WINDOWS do
			Sea.util.hook("ChatFrame"..i,"LHCP_ChatFrame_OnEvent","hide","OnEvent");
		end
	else
		local ChatFrame_Orig_OnEvent = ChatFrame_OnEvent;
		ChatFrame_OnEvent = function(event)
			if LHCP_ChatFrame_OnEvent(event) then
				return ChatFrame_Orig_OnEvent(event);
			else
				return;
			end
		end
	end
end

---
-- 
---
function LHCP_HookEvents(table)
	if not table then
		table = LHCP_events;
	end
	
	for key,value in pairs(table) do
		this:RegisterEvent(key);
	end
end

---
-- 
---
function LHCP_UnhookEvents(table)
	if not table then
		table = LHCP_events;
	end
	
	for key,value in pairs(LHCP_events) do
		this:UnregisterEvent(key);
	end
end

---
--- Intercept says
---


function LHCP_HandleTextEmote()
	if not LHCP_isEnabled then
		return nil;
	end
	
	if not LHCP_IgnoreSpam()
	  and not (LHCP_RaidIgnore() and LHCP_PlayerIsInRaidInstance()) then
		return;
	end
	
	LHCP_Debug("HandleTextEmote ",event," ",arg1);
	
	
	local now = time();
	if LHCP_lastEmote and now >= LHCP_emoteTime then
		LHCP_lastEmote = nil;
	end
	
	LHCP_emoteTime = now + 5;
	
	if LHCP_lastEmote and string.find(arg1,LHCP_lastEmote) then

		LHCP_skipEvent = event;
		-- filter this text from the screen
		LHCP_skipText = arg1;
		LHCP_Debug("Skip ",event," ",arg1);
	else
		for _,emote in ipairs(LeeroyHillCatsPower_emotes) do
			if string.find(arg1,emote) then
				LHCP_lastEmote = emote;
				LHCP_Debug("Last emote ",emote);
				return;
			end
		end
	end
end

-- returns new file location if it has been better established
function LHCP_PlaySoundFile(file)
	if not LHCP_isEnabled then
		return nil;
	end
	
	if not file then
		return nil;
	end

	LHCP_Debug("Playing ",file);
	if PlaySoundFile(file) == 1 then
		return file;
	end
	
	local newfile;
	for i,ext in ipairs(LeeroyHillCatsPower_extensions) do
		newfile = file..ext;
		LHCP_Debug("Playing ",newfile);
		if PlaySoundFile(newfile) == 1 then
			return newfile;
		end
	end
	
	LHCP_Debug("Error playing ",file);
	return file;
end

function LHCP_SayOutgoingEmote(key)
	local emote = LeeroyHillCatsPower_data[key];

	if emote == nil then return; end
	
	SendChatMessage(emote["text"], "SAY");
end

function LHCP_PlayIncomingText()
	if not LHCP_isEnabled then
		return nil;
	end
	
	LHCP_Debug("PlayIncomingText ",event," ",arg1);
	
	local key = LHCP_phrases[arg1];
	
	LHCP_PlayIncomingEmote(key,event,arg1,arg2);
end

function LHCP_PlayIncomingEmote(key,event,arg1,arg2)
	local emote = LeeroyHillCatsPower_data[key];
	local selfplay = arg1 == nil; -- indicates self-play
	
	if emote == nil then return; end
	
	local emoteinfo = ChatTypeInfo["EMOTE"];
	
	if not selfplay then
		LHCP_skipEvent = event; -- skip the "say"
		LHCP_skipText = arg1;
		
		LHCP_Debug("Skipping ",event," ",arg1);
	end
	
	if LHCP_RaidIgnore() and LHCP_PlayerIsInRaidInstance() then
		LHCP_Debug("Raid ignoring ",key);
		return;
	end
	
	if emote["msg"] then
		if selfplay or LHCP_PlayEmotes() then
			LHCP_Print(arg2..emote["msg"],
			  emoteinfo.r, emoteinfo.g, emoteinfo.b);
		end
	end
	
	if emote["emote"] then
		if selfplay or LHCP_PlayEmotes() then
			DoEmote(emote["emote"]);
		end
	end

	if emote["file"] then
		if not LHCP_IsThrottled() and LHCP_PlaySounds() then
			if emote["nsfw"] and not LHCP_PlayNsfw() then
				LHCP_Debug("Not playing NSFW emote: ",key);
				return;
			end
			
			-- compare text to the opposing faction text
			-- if match, then this emote came from an enemy
			if (not LHCP_PlayEnemy())
			  and emote[LHCP_factionText] == arg1 then
			  	LHCP_Debug("Not playing enemy emote: ",key);
				return;
			end
			
		  	emote["file"] = LHCP_PlaySoundFile(emote["file"]);
			LHCP_SetThrottled(2)
			-- minimum 2 seconds between all emotes
		end
	end
end

function LHCP_IsCensored(key)
	local emote = LeeroyHillCatsPower_data[key]
	return not LHCP_PlayNsfw() or emote and emote["nsfw"]
end

function LHCP_IsThrottled()
	return time() < LHCP_timeTilNext
end

function LHCP_SetThrottled(duration)
	LHCP_timeTilNext = time() + duration
end

function LHCP_PlayerIsInRaidInstance()
	local zone = GetRealZoneText();

	return LeeroyHillCatsPower_ignoreZones[zone];
end

function LHCP_DoEgg()
	LHCP_Debug("Egg ",event," ",arg1);
	
	local egg = LHCP_eggs[event];
	
	if not egg or not arg1 then return; end
	
	for i,sound in pairs(egg) do
		if egg["text"] and string.find( arg1, egg["text"]) then
			egg["file"] = LHCP_PlaySoundFile(egg["file"]);
		end
	end
end

function LHCP_OnEvent()
	if not LHCP_phrases then
		LHCP_Load();
	end
	
	LHCP_HandleEvent();
end

function LHCP_HandleEvent()
	if LHCP_events then
		local func = LHCP_events[event];
		if func ~= nil then
			func();
		end
	end
end

---
--- Replace chat frame check to remove says
---

function LHCP_ChatFrame_OnEvent()
	if LHCP_skipEvent and LHCP_skipEvent == event
		and LHCP_skipText == arg1 then
		-- we got the event we want to skip
		LHCP_Debug("Skipped ",arg1);
		return false;
	else
		return true;
	end
end

---
--- Slash Commands
---

---
-- Debug logging, uses varargs to avoid concatenation expense when debug is disabled
---
function LHCP_Debug(...)
	LHCP_Print(arg,nil,nil,nil,true,true);
end

function LHCP_PrintKeys(text,r,g,b,delim)
	if not delim then
		delim = " ";
	end
	LHCP_Print(text,r,g,b,true,false,delim,"key");
end

---
-- 
---
function LHCP_Print(text,r,g,b,oneline,dodebug,delim,item)
	if not text then
		return;
	end
	
	if dodebug and not LHCP_debug then
		return;
	end
	
	local out = "";

	if not delim then
		delim = "";
	end
	
	local i = 1;
	if type(text) == "table" then
		for k,v in pairs(text) do
			local msg = v;
			if item == "key" then
				msg = k;
			end

			if oneline then
				-- TODO how to unroll the entire table at once?
				if i == 1 then
					out = out..msg;
				else
					out = out..delim..msg;
				end
			else
				DEFAULT_CHAT_FRAME:AddMessage(delim..msg,r,g,b);
			end
			i = i+1;
		end
		
		if oneline then
			DEFAULT_CHAT_FRAME:AddMessage(out,r,g,b);
		end
	else
		DEFAULT_CHAT_FRAME:AddMessage(delim..text,r,g,b);
	end
end

function LHCP_List(arg)
	if arg == "all" then
		LHCP_ListAll();
	else
		LHCP_ListCategory(arg);
	end
end

function LHCP_ListAll()
	-- list all events
	if not LHCP_actionList then
		LHCP_actionList = {};
		
		local allCmds = {};
		for key,value in pairs(LeeroyHillCatsPower_data) do
			
			table.insert(allCmds,key);
			
			local i = 1;
			local cmd = "cmd"..i;
			while value[cmd] do
				table.insert(allCmds,value[cmd]);
				i = i+1;
				cmd = "cmd"..i;
			end
		end
		
		table.sort(allCmds);
		
		LHCP_actionList = {};
		local str = "";
		for i,value in ipairs(allCmds) do
			if string.len(str) + string.len(value) > 50 then
				table.insert(LHCP_actionList,str);
				str = "";
			end

			str = str .." ".. value;
		end
		table.insert(LHCP_actionList,str);
	end
	
	LHCP_Print("LeeroyHillCatsPower!");
	LHCP_Print(LHCP_actionList);
	LHCP_Print(LeeroyHillCatsPower_new);
end


function LHCP_main()
	LHCP_Print(LeeroyHillCatsPower_credits);
	LHCP_Print(LeeroyHillCatsPower_about);
	LHCP_Print(LeeroyHillCatsPower_new);
end

function LHCP_help(msg)
	LHCP_Print(LeeroyHillCatsPower_about);
	LHCP_Print(LeeroyHillCatsPower_help);
	LHCP_Print(LeeroyHillCatsPower_new);
end

-- ignore emotes, but participate
function LHCP_quiet(quiet)
	LHCP_ignoreMode = bit.bor(LHCP_ignoreMode,LHCP_IGNORE_SPAM,LHCP_IGNORE_SOUNDS)
	LHCP_ignoreMode = bit.band(LHCP_ignoreMode,bit.bnot(LHCP_IGNORE_EMOTES))
	_= quiet or LHCP_Print("LHCP ignoring sounds")
	LHCP_OnUpdateStatus()
	
	if LHCP_globalIgnoreMode then
		LHCP_globalIgnoreMode = LHCP_ignoreMode;
	end

	LHCP_nextToggle = LHCP_toggle["quiet"];
end

-- ignore all emote requests
function LHCP_ignore(quiet)
	LHCP_ignoreMode = bit.bor(LHCP_ignoreMode,LHCP_IGNORE_SPAM,LHCP_IGNORE_SOUNDS,LHCP_IGNORE_EMOTES);
	
	_= quiet or LHCP_Print("LHCP ignoring all events");
	LHCP_OnUpdateStatus()

	if LHCP_globalIgnoreMode then
		LHCP_globalIgnoreMode = LHCP_ignoreMode;
	end

	LHCP_nextToggle = LHCP_toggle["ignore"];
end

function LHCP_PlaySounds()
	return bit.band(LHCP_ignoreMode,LHCP_IGNORE_SOUNDS) == 0;
end

function LHCP_PlayEmotes()
	return bit.band(LHCP_ignoreMode,LHCP_IGNORE_EMOTES) == 0;
end

function LHCP_PlayEnemy()
	return bit.band(LHCP_ignoreMode,LHCP_IGNORE_ENEMY) == 0;
end

-- unignore all
function LHCP_unignore(quiet)
	-- preserve all other bits
	LHCP_ignoreMode = bit.band(LHCP_ignoreMode,
	  bit.bnot(bit.bor(LHCP_IGNORE_SPAM,LHCP_IGNORE_SOUNDS,LHCP_IGNORE_EMOTES)));
	
	_= quiet or LHCP_Print("LHCP not ignoring anything");
	LHCP_OnUpdateStatus()

	if LHCP_globalIgnoreMode then
		LHCP_globalIgnoreMode = LHCP_ignoreMode;
	end

	LHCP_nextToggle = LHCP_toggle["unignore"];
end

-- ignore pvp emotes
function LHCP_pvp(quiet)
	if LHCP_PlayEnemy() then
		LHCP_ignoreMode = bit.bor(LHCP_ignoreMode,LHCP_IGNORE_ENEMY); -- enable ignore enemy
		_= quiet or LHCP_Print("LHCP ignoring cross-faction events");
	else
		LHCP_ignoreMode = bit.band(LHCP_ignoreMode,bit.bnot(LHCP_IGNORE_ENEMY)); -- disable ignore enemy
		_= quiet or LHCP_Print("LHCP playing cross-faction events");
	end
	LHCP_OnUpdateStatus()

	if LHCP_globalIgnoreMode then
		LHCP_globalIgnoreMode = LHCP_ignoreMode;
	end
end

function LHCP_nospam(quiet)
	LHCP_ignoreMode = bit.bor(LHCP_ignoreMode,LHCP_IGNORE_SPAM)
	LHCP_ignoreMode = bit.band(LHCP_ignoreMode,
	  bit.bnot(bit.bor(LHCP_IGNORE_SOUNDS,LHCP_IGNORE_EMOTES)))
	_= quiet or LHCP_Print("LHCP ignoring spammed emotes")
	LHCP_OnUpdateStatus()

	if LHCP_globalIgnoreMode then
		LHCP_globalIgnoreMode = LHCP_ignoreMode;
	end

	LHCP_nextToggle = LHCP_toggle["nospam"];
end

function LHCP_IgnoreSpam()
	return bit.band(LHCP_ignoreMode,LHCP_IGNORE_SPAM) ~= 0;
end

function LHCP_Nsfw(quiet)
	LHCP_ignoreMode = bit.bor(LHCP_ignoreMode,LHCP_PLAY_NSFW);
	_= quiet or LHCP_Print("LHCP playing NSFW emotes");
	LHCP_OnUpdateStatus()

	if LHCP_globalIgnoreMode then
		LHCP_globalIgnoreMode = LHCP_ignoreMode;
	end
end

function LHCP_Sfw(quiet)
	LHCP_ignoreMode = bit.band(LHCP_ignoreMode,bit.bnot(LHCP_PLAY_NSFW));
	_= quiet or LHCP_Print("LHCP not playing NSFW emotes");
	LHCP_OnUpdateStatus()

	if LHCP_globalIgnoreMode then
		LHCP_globalIgnoreMode = LHCP_ignoreMode;
	end
end

function LHCP_PlayNsfw()
	return bit.band(LHCP_ignoreMode,LHCP_PLAY_NSFW) ~= 0;
end

-- privately play it
function LHCP_PrivatePlay(arg1,arg2)
	if arg1 == "random" then
		arg1 = LHCP_GetRandomEmote(arg2);
		
		if arg2 and not arg1 then
			LHCP_Print("Bad category: "..arg2);
			return;
		end

		LHCP_Print("/"..arg1);
	end
	
	LHCP_Debug("Private playing ",arg1);
	LHCP_PlayIncomingEmote(arg1,"CHAT_MSG_SAY",nil,UnitName("player"));
end

-- privately play emote with NO chat spam or emoting
function LHCP_Preview(arg1,arg2)
	if arg1 == "random" then
		arg1 = LHCP_GetRandomEmote(arg2);
		
		if arg2 and not arg1 then
			LHCP_Print("Bad category: "..arg2);
			return;
		end

		LHCP_Debug("/",arg1);
	end
	
	if not arg1 then
		return
	end
	
	local emote = LeeroyHillCatsPower_data[arg1];
	if not emote then
		LHCP_Print(arg1.." not found")
	end
	
	LHCP_Debug("Playing ",arg1);
	LHCP_PlaySoundFile(emote["file"]);
end

function LHCP_ToggleIgnore(quiet)
	local f = LHCP_cmds[LHCP_nextToggle];
	
	LHCP_Debug(LHCP_GetStatus(),"->",LHCP_toggle[LHCP_GetStatus()]," next: ",LHCP_nextToggle," ",type(f));
	
	f(quiet);
end

function LHCP_OnUpdateStatus()
	if LeeroyFu then
		LeeroyFu:UpdateText()
		LeeroyFu:Update()
	end
end

function LHCP_GetStatus()
	if not LHCP_PlayEmotes() then
		return "ignored"
	elseif not LHCP_PlaySounds() then
		return "quiet"
	elseif LHCP_IgnoreSpam() then
		return "nospam"
	end
	
	if LHCP_RaidIgnore() then
		return "raid"
	end

	if not LHCP_PlayEnemy() then
		return "PvP"
	end
	
	if LHCP_PlayNsfw() then
		return "NSFW"
	end
	
	return ""
end

function LHCP_ShowStatus()
	if not LHCP_PlayEmotes() then
		LHCP_Print("LHCP is ignored",0,1,0);
	elseif not LHCP_PlaySounds() then
		LHCP_Print("LHCP is in quiet mode",0,1,0);
	end
	
	if not LHCP_PlayEnemy() then
		LHCP_Print("LHCP is ignoring enemy emotes",0,1,0);
	end
	
	if LHCP_IgnoreSpam() then
		LHCP_Print("LHCP is ignoring spammed emotes",0,1,0);
	end
	
	if LHCP_PlayNsfw() then
		LHCP_Print("LHCP will play NSFW sounds",0,1,0);
	end
	
	if LHCP_RaidIgnore() then
		LHCP_Print("LHCP will ignore all sounds inside raid instances",0,1,0);
	end
end

function LHCP_ListCategory(arg)
	if not arg then
		LHCP_PrintKeys(LeeroyHillCatsPower_categories);
	else
		if not LeeroyHillCatsPower_categories[arg] then
			LHCP_Print("Bad category: "..arg);
			return;
		end
		
		local table = LeeroyHillCatsPower_categories[arg]["data"];
		
		if not table or getn(table) < 1 then
			LHCP_Print("Empty category: "..arg);
			return;
		end

		LHCP_Print(table,nil,nil,nil,true,false," ");
	end
end

function LHCP_PrintCategories()
	for k,v in pairs(LeeroyHillCatsPower_categories) do
		if v and v["help"] then
			if string.len(k) > 11 then
				LHCP_Print(k);
				LHCP_Print("            "..v["help"]);
			else
				LHCP_Print(k..
				  string.rep(" ",12 - string.len(k))..
				  v["help"]);
			end
		end
	end
end

function LHCP_PrintInfo(arg)
	if not arg then
		LHCP_Print("Supply info argument: /lhcp info leeroy");
		return;
	end
	
	local data = LeeroyHillCatsPower_data[arg];
	local eegg = LeeroyHillCatsPower_eastereggs[arg];
	local category = LeeroyHillCatsPower_categories[arg];

	if (not data) and (not eegg) and not category then
		LHCP_Print("Unknown item: "..arg);
		return;
	end

	if data then
		LHCP_Print("Emote: "..arg);
		LHCP_PrintTable(data);
	end

	if eegg then
		LHCP_Print("Easteregg: "..arg);
		LHCP_PrintTable(eegg);
	end

	if category then
		LHCP_Print("Category: "..arg);
		LHCP_PrintTable(category);
	end
end

function LHCP_PrintTable(arg)
	for k,v in pairs(arg) do
		if type(v) == "table" then
			local n = getn(v);
			LHCP_Print("  "..k.." = ("..n.." items)");
			if n < 6 then
				LHCP_Print(v,nil,nil,nil,false,false,"   ");
			end
		else
			if not v then
				v = "nil";
			end
			LHCP_Print("  "..k.." = "..v);
		end
	end
end

function LHCP_Random(arg,quiet)
	local key = LHCP_GetRandomEmote(arg);
	
	if arg2 and not arg1 then
		_= quiet or LHCP_Print("Bad category: "..arg2);
		return;
	end
	
	_= quiet or LHCP_Print("/"..key);
	
	LHCP_SayOutgoingEmote(key);
end

function LHCP_GetRandomEmote(arg)
	if not arg then
		return LHCP_GetIRandom(LHCP_emoteList);
	else
		if not LeeroyHillCatsPower_categories[arg] then
			return nil;
		end
		
		local table = LeeroyHillCatsPower_categories[arg]["data"];
		if not table then return nil; end
		
		return LHCP_GetIRandom(table);
	end
end

function LHCP_GetIRandom(table)
	if not table then
		error("Cannot choose random element of nil talbe");
	end
	
	local n = getn(table);
	
	if n == 1 then
		return 1;
	end
	
	LHCP_Debug("getn(table) returned (",type(n),") ",n);
	
	n = math.random(n); -- rand from 1,n
	
	return table[n];
end

function LHCP_ToggleGlobalPrefs(quiet)
	if LHCP_globalIgnoreMode then
		_= quiet or LHCP_Print("LHCP global preferences disabled")
		LHCP_globalIgnoreMode = false
	else
		_= quiet or LHCP_Print("LHCP global preferences enabled")
		LHCP_globalIgnoreMode = LHCP_ignoreMode
	end
end

function LHCP_RaidIgnore()
	return bit.band(LHCP_ignoreMode,LHCP_IGNORE_RAID) ~= 0;
end

function LHCP_ToggleRaidIgnore(quiet)
	if LHCP_RaidIgnore() then
		-- clear raid ignore
		LHCP_ignoreMode = bit.band(LHCP_ignoreMode,bit.bnot(LHCP_IGNORE_RAID));
		_= quiet or LHCP_Print("LHCP not ignoring in raid")
	else
		LHCP_ignoreMode = bit.bor(LHCP_ignoreMode,LHCP_IGNORE_RAID);
		_= quiet or LHCP_Print("LHCP ignoring sounds in raid")
	end
	LHCP_OnUpdateStatus()
end

function LHCP_EnableDebug(quiet)
	if(LHCP_debug) then
		LHCP_debug = false
		_= quiet or LHCP_Print("Debug disabled")
	else
		LHCP_debug = true
		_= quiet or LHCP_Print("Debug enabled")
	end
end

LHCP_cmds = {
	["list"] = LHCP_List,
	[""] = LHCP_main,
	["help"] = LHCP_help,
	["quiet"] = LHCP_quiet,
	["ignore"] = LHCP_ignore,
	["unignore"] = LHCP_unignore,
	["pvp"] = LHCP_pvp,
	["nospam"] = LHCP_nospam,
	["play"] = LHCP_PrivatePlay,
	["preview"] = LHCP_Preview,
	["nsfw"] = LHCP_Nsfw,
	["sfw"] = LHCP_Sfw,
	["toggle"] = LHCP_ToggleIgnore,
	["global"] = LHCP_ToggleGlobalPrefs,
	["raidignore"] = LHCP_ToggleRaidIgnore,
	["debug"] = LHCP_EnableDebug,
	["status"] = LHCP_ShowStatus,
	["menu"] = LHCP_PrintCategories,
	["info"] = LHCP_PrintInfo,
	["random"] = LHCP_Random,
}

LHCP_toggle = { -- TODO toggle needs improvement
	[""] = "nospam",
	["ignored"] = "unignore",
	["NSFW"] = "nospam",
	["PvP"] = "nospam",
	["raid"] = "nospam",
	["nospam"] = "quiet",
	["quiet"] = "ignore",
	["ignore"] = "unignore",
	["unignore"] = "nospam",
}

LHCP_nextToggle = nil

function lhcp_command(msg)
	local argv = {}
	for s in string.gfind(msg, "%w+") do
		table.insert(argv,s);
	end
	
	local cmd = table.remove(argv,1);
	
	if not cmd then
		cmd = "";
	end
	
	local func = LHCP_cmds[cmd];
	
	if func ~= nil then
		func(unpack(argv));
	else
		LHCP_SayOutgoingEmote(cmd);
	end

end

-- our main slash command
SlashCmdList["LHCP"] = lhcp_command;
SLASH_LHCP1 = "/lhcp";

